import 'package:flutter/material.dart';
import 'base_route_widget_state.dart';
import 'dart:math' as math;
import './base_tab_bar_view_item_state.dart';

/// PageView 子项抽象状态,仅支持直接使用PageView()构造
/// 提供了生命周期监听
/// 需要在MaterialApp.navigatorObservers中注册BaseRouteState.baseRouteStateObserver
abstract class BasePageViewItemState<T extends StatefulWidget>
    extends BaseRouteWeigetState<T> with AutomaticKeepAliveClientMixin {
  PageController _oldController;
  @override
  bool get wantKeepAlive => true;

  @override
  void initState() {
    super.initState();
    if (wantKeepAlive) {
      Future.delayed(Duration(milliseconds: 300)).then((value) {
        _controllerListener();
      });
    }
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    if (wantKeepAlive && _oldController != getPageController()) {
      _addListener();
    }
  }

  @override
  void didUpdateWidget(covariant T oldWidget) {
    super.didUpdateWidget(oldWidget);
    _addListener();
  }

  void _addListener() {
    if (wantKeepAlive && _oldController != getPageController()) {
      _oldController = getPageController();
      getPageController().addListener(_controllerListener);
      _addParentListener();
    }
  }

  void _addParentListener() {
    BasePageViewItemState pstate =
        context.findAncestorStateOfType<BasePageViewItemState>();
    if (pstate != null && pstate.getPageController() != null) {
      pstate.getPageController().addListener(() {
        getPageController().notifyListeners();
      });
    }
    BaseTabBarViewItemState tstate =
        context.findAncestorStateOfType<BaseTabBarViewItemState>();
    if (tstate != null && tstate.getTabController() != null) {
      tstate.getTabController().addListener(() {
        getPageController().notifyListeners();
      });
    }
  }

  void _controllerListener() {
    if (isCurrent()) {
      if (!isResumed()) {
        onResume();
      }
    } else {
      if (isResumed()) {
        onPause();
      }
    }
  }

  PageController getPageController() {
    PageView pageView = _getPageView();
    return pageView.controller;
  }

  PageView _getPageView() {
    PageView pageView = context.findAncestorWidgetOfExactType<PageView>();
    return pageView;
  }

  List<Widget> _getTabs() {
    PageView pageView = _getPageView();
    SliverChildListDelegate delegate =
        pageView.childrenDelegate as SliverChildListDelegate;
    return delegate.children;
  }

  bool isCurrent() {
    if (isParentCurrent()) {
      List<ScrollPosition> positions = getPageController().positions;
      for (int i = 0, len = positions.length; i < len; i++) {
        var position = positions[i];
        var pixels = getPageFromPixels(
            position.pixels.clamp(
                position.minScrollExtent, position.maxScrollExtent) as double,
            position.viewportDimension);
        int index = pixels.round();
        var currentWidget = _getTabs()[index];
        /*if (currentWidget
            is extended.NestedScrollViewInnerScrollPositionKeyWidget) {
          currentWidget = (currentWidget
                  as extended.NestedScrollViewInnerScrollPositionKeyWidget)
              .child;
        }*/
        return widget == currentWidget;
      }
    }

    return false;
  }

  bool isParentCurrent() {
    BasePageViewItemState pstate =
        context.findAncestorStateOfType<BasePageViewItemState>();
    BaseTabBarViewItemState tstate =
        context.findAncestorStateOfType<BaseTabBarViewItemState>();
    if (pstate != null) {
      return pstate.isCurrent();
    }
    if (tstate != null) {
      return tstate.isCurrent();
    }

    return true;
  }

  double getPageFromPixels(double pixels, double viewportDimension) {
    var viewportFraction = getPageController().viewportFraction;
    double _initialPageOffset =
        math.max(0, viewportDimension * (viewportFraction - 1) / 2);
    final double actual = math.max(0.0, pixels - _initialPageOffset) /
        math.max(1.0, viewportDimension * viewportFraction);
    final double round = actual.roundToDouble();
    if ((actual - round).abs() < 1e-10) {
      return round;
    }
    return actual;
  }

  @override
  void onPageShow() {
    if (wantKeepAlive && isCurrent() && !isResumed()) {
      super.onPageShow();
    }
  }

  @override
  void onPageHide() {
    if (wantKeepAlive && isCurrent() && isResumed()) {
      super.onPageHide();
    }
  }

  @override
  void onBackground() {
    if (wantKeepAlive && isCurrent()) {
      super.onBackground();
    }
  }

  @override
  void onForeground() {
    if (wantKeepAlive && isCurrent()) {
      super.onForeground();
    }
  }
}
