library xapp_utils;

import 'dart:convert';
import 'dart:io';
import 'package:flutter/services.dart';
import 'package:flutter/services.dart' show rootBundle;
import 'package:device_info/device_info.dart';
import 'package:android_device/android_device.dart';
import 'package:crypto/crypto.dart' as crypto;
import 'package:flutter/material.dart';
import 'package:flutter_secure_storage/flutter_secure_storage.dart';
import 'package:package_info/package_info.dart';
import 'loading_dialog_utils.dart';
import 'package:flutter_native_toast/flutter_native_toast.dart';

class XAppUtils {
  static XAppUtils _instance;

  static XAppUtils getInstance() {
    if (_instance == null) {
      _instance = new XAppUtils._();
    }
    return _instance;
  }

  double _mediaWidth;
  double _mediaHeight;
  double _adaptWidth;
  String _deviceId;
  String _deviceType;
  String _osVersion;
  String _appVersion;
  String _appBuildNumber;
  String _idfaID;

  XAppUtils._();

  /// 初始化
  /// [context] 上下文
  /// [uiWidth] UI尺寸宽度
  void init({@required BuildContext context, double uiWidth = 375}) async {
    if (_adaptWidth == null || _adaptWidth == 0.0) {
      Size size = MediaQuery.of(context).size;
      _mediaWidth = size.width;
      _mediaHeight = size.height;
      _adaptWidth = _mediaWidth / uiWidth;
      if(_appVersion == null){
      var packageInfo = await PackageInfo.fromPlatform();
      _appBuildNumber = packageInfo.buildNumber;
      _appVersion = packageInfo.version;
      }
    }
  }

  /// 获取屏幕宽度
  double getMediaWidth() {
    return _mediaWidth;
  }

  /// 获取屏幕高度
  double getMediaHeight() {
    return _mediaHeight;
  }

  double getAdaptWidth() {
    return _adaptWidth;
  }
  /// 根据UI单位转换成rpx
  double rpx(num px) {
    return double.parse((px * getAdaptWidth()).toStringAsFixed(6));
  }

  LoadingDialog showLoading([String msg = '请稍后']) {
    return LoadingDialogUtils.getInstance().show(msg: msg);
  }

  /// sha1编码
  String sha1(String data) {
    var bytes = utf8.encode(data);
    var digest = crypto.sha1.convert(bytes);
    return digest.toString();
  }

  /// md5编码
  String md5(String data) {
    var bytes = utf8.encode(data);
    var digest = crypto.md5.convert(bytes);
    return digest.toString();
  }

  /// 获取rsa密钥
  Future<String> getRSAKeyByAsset(String file) async {
    var keyStr = await rootBundle.loadString(file);
    return keyStr;
  }

  /// 获取设备ID
  Future<String> getDeviceId() async {
    if (_deviceId == null) {
      if (Platform.isIOS) {
        var info = await DeviceInfoPlugin().iosInfo;
        _deviceId = info.identifierForVendor;
      } else {
        _deviceId = await AndroidDevice.getDeviceId();
      }
    }

    return _deviceId;
  }

  /// 获取设备类型,如: iPod7,1
  Future<String> getDeviceType() async {
    if (_deviceType == null) {
      if (Platform.isAndroid) {
        AndroidDeviceInfo info = await DeviceInfoPlugin().androidInfo;
        _deviceType = info.model;
      } else {
        IosDeviceInfo info = await DeviceInfoPlugin().iosInfo;
        _deviceType = info.model;
      }
    }
    return _deviceType;
  }

  /// 获取系统类型,如: ios、android
  String getOsType() {
    return Platform.operatingSystem;
  }

  /// 获取系统版本号,如: 10
  Future<String> getOsVersion() async {
    if (_osVersion == null) {
      if (Platform.isAndroid) {
        AndroidDeviceInfo info = await DeviceInfoPlugin().androidInfo;
        _osVersion = info.version.release;
      } else {
        IosDeviceInfo info = await DeviceInfoPlugin().iosInfo;
        _osVersion = info.systemVersion;
      }
    }

    return _osVersion;
  }

  /// 获取app版本号,如: 1.0.0
  String getAppVersion() {
    return _appVersion;
  }
  
  String getAppBuildNumber(){
    return _appBuildNumber;
  }

  /// 设置剪贴板内容
  Future<void> setClipboardText(String text) {
    return Clipboard.setData(ClipboardData(text: text));
  }

  /// 获取剪贴板内容
  Future<String> getClipboardText() async {
    ClipboardData data = await Clipboard.getData(Clipboard.kTextPlain);
    if (data == null) {
      return '';
    }
    return data.text ?? '';
  }

  Future<String> getIDFAID() async {
    if (_idfaID != null) {
      return _idfaID;
    }
    //读取存储的设备id,没有id就创建一个
    //设备唯一id
    _idfaID = await FlutterSecureStorage().read(key: 'kIDFAID');
    if (_idfaID == null || _idfaID.contains('000000000000')) {
      if (Platform.isIOS) {
        var info = await DeviceInfoPlugin().iosInfo;
        // idfaID = await pangle.getIdfaString();
        _idfaID = info.identifierForVendor;
      } else {
        _idfaID = await AndroidDevice.getDeviceId();
      }
      FlutterSecureStorage().write(key: 'kIDFAID', value: _idfaID);
    }
    return _idfaID;
  }

  void toast(String msg) {
    FlutterNativeToast.showToast(msg);
  }
}

extension SizeExtension on num {
  double get rpx => XAppUtils.getInstance().rpx(this);
  double get w => XAppUtils.getInstance().rpx(this);
  double get nsp => XAppUtils.getInstance().rpx(this);
}

Color rgba(int r, int g, int b, double a) => Color.fromRGBO(r, g, b, a);
